@@ -14,6 +14,57 @@ import wx.lib.inspection
 from read_data import read_data
 import pandas as pd
 import wx.richtext as rt
+import time
+
+
+## Main script that runs the GUI program for HEPPDAP. Upon initial run, the app object is created
+## (MainApp) which inherits from wx.App class. Within the app, the main frame is created (MainFrame)
+## which inherits from wx.Frame class. The app is the entire program while the frame represents the
+## window in which things can be placed. MainFram has two children: apanel ("a" standing for analysis), which 
+## holds the PlotNoteBook object. This PlotNoteBook takes up the entire right hand side of the frame (window)
+## it has a method called "add" which allows additional plots to be added as additional pages to the PlotNoteBook.
+## The other child is the panel, this can be thought of as the actual control panel as it holds all the buttons and
+## inputs that allow the user to run the program and plot data. All controls are decendants of this class
+## User selects file using the browse button, once analyze is clicked, the read_data method is run. This takes
+## the raw data, unpacks it and returns an array that indexes the evetns as followed: 
+## event = read_data("file.dat")[event_number][channel_index] where event is a pandas array that contains the 
+## voltages and times. This allows for easy plotting as pandas and matplotlib work well with eachother
+## This program reads the output of read_data and interates through the first event of each channel.
+## If data is detected in the array, the associated channel index is added to a list of "active channels"
+## This is just a way for the program to know which channels have data and which ones do not. For example,
+## if channel 1 had data, channel 2 did not have data and channel 3 had data and all data came from myfile.dat,
+## read_data("myfile.dat")[0][0] and read_data("myfile.dat")[0][2] would have data while read_data("myfile.dat)[0][1]
+## would be an empty array. This is done so that the user can only select from channels that actually have data.
+## In other words, buttons/settings for all channels will always be created for each file run, however, if a channel does
+## not have data in it, the program will hide all controls that interact with that channel so that the user is not able 
+## is access controls that correspond to an empty channel
+## User can filter (or apply post-analysis triggers) such as specifcying a minimum amplitude of each channel and 
+## maxmimum timing difference between 2 channels. All filtration parameters use "AND" logic. Once the user specifies
+## filtration parameters, the program iterates through each event for each selected channel. If the first criterion
+## is satisfied, the next criterion is evaluated, this continues until one of two things happen:
+## 1) The event fails to meet the required criterion:
+##    Program will then go to the next event and restart the filtration process
+## 2) All criterion are passed:
+##    Program plots the event with the selected channels
+## If the user is viewing the most recently generated plot, upon clicking the "Next" button, 
+## the next event (that satisfies all (if any) filtration criterion) is plotted. The program knows if the user is 
+## on the last page (i.e. most recently plotted event) by getting the index of the page and comparing it to the total
+## number of pages in the PlotNoteBook. If the user is viewing an event that is not the last, the "Next" button will 
+## simply advance the tab (page) to the next plot that has already been generated. The "Previous" button always returns the plot to
+## the previous tab, no matter where the user is (unless, of course, the user is viewing the first tab (event) plotted).
+## If the user wants to analyze a new file, he/she must simply click "Browse", select the new file, and click "Analyze" 
+## THe "Analyze" button can be thought of as a 'reset button', upon being clicked it will clear the entire PlotNoteBook and 
+## reset all indexes being used by the program to 0. It basically starts the program from the beginning.
+
+
+#All characteristics (i.e. rise time, fall time, amplitude, etc) are obtained from using the methdod calc_stats() in read_data()
+## Returns associated array that is indexed by [channel_number][characteristic][event_number], the PlotNoteBook has a "on_tab_changed" event 
+## listener that is always running. It will update the displayed characteristics based on the currenlty viewed page in the PlotNoteBook
+## THis program makes extensive use of numpy, matplotlib and wxPython, before really digging deep into this code, it is
+## highly recommended that a working knowledge of matplotlib and wxPython (especially) is gained.
+
+
+
 
 
 class MainApp(wx.App):
@@ -35,16 +86,42 @@ class MainFrame(wx.Frame):
         self.InitFrame()
 
     def InitFrame(self):
+        menubar = wx.MenuBar()
+        fileMenu = wx.Menu() 
         # Make sizer and add panels to sizer to for auto-resizing
         sizer=wx.BoxSizer(wx.HORIZONTAL)
         self.apanel = PlotNotebook(self, id=-1, pos=(100,100), size=(300,300))
         self.panel = MainPanel(self, id=wx.ID_ANY, pos=(0,100), size=(100,100))
         sizer.Add(self.panel, 1, wx.EXPAND|wx.ALL, 5)
+
+        #Build Menu
+        oid = wx.NewId()
+        aid = wx.NewId()
+        #Open Option Under File
+        openitem = fileMenu.Append(wx.ID_ANY, "Open\tCtrl+O", "Open") 
+        #Bind Item
+        self.Bind(wx.EVT_MENU, self.panel.onOpenFile, openitem)
+        #Bind Keyboard Shortcut
+        self.Bind(wx.EVT_MENU, self.panel.onOpenFile, id=oid)
+        #Analyze Item
+        analyzeitem = fileMenu.Append(wx.ID_ANY, "Analyze\tReturn", "Analyze")
+        #Bind Item
+        self.Bind(wx.EVT_MENU, self.panel.analyze, analyzeitem)
+        #Bind Keyboard Shortcut
+        self.Bind(wx.EVT_MENU, self.panel.analyze, id=aid)
+
+        menubar.Append(fileMenu, '&File')
+        self.SetMenuBar(menubar)
         sizer.Add(self.apanel, 3, wx.ALIGN_TOP|wx.EXPAND, 0)
+        self.accel_tbl = wx.AcceleratorTable([(wx.ACCEL_CTRL, ord('O'), oid)])
+        self.SetAcceleratorTable(self.accel_tbl)
         self.SetSizer(sizer)
 
 
 
+
+
+
 class PlotNotebook(wx.Panel):
     #This is the object that handles the plotting of data
     def __init__(self, parent, id, pos, size):
@@ -61,8 +138,10 @@ class PlotNotebook(wx.Panel):
      
     #Method to add a new tab (plot) to the notebook
     def add(self, name="plot"):
+        #Get the actual plot
         page = Plot(self.nb)
         page.name=name
+        # Add plot to the PloteNoteBook as an additional page
         self.nb.AddPage(page, name)
         return page.figure
 
@@ -75,13 +154,13 @@ class PlotNotebook(wx.Panel):
                 if channel.IsChecked():
                     bindex = board.index(channel)        
 
-        print("Channel: %s" % bindex)
+        #print("Channel: %s" % bindex)
         #Get index from the name of the current tab
         index = int(self.nb.GetCurrentPage().name.split()[1]) - 1
         mainpanel.index = index 
         #Grab characteristics from the data
         mainpanel.peaktimeval.SetLabel(str(mainpanel.data.stats['single_channel'][bindex]["peak_time"][index]))
-        mainpanel.risetimeval.SetLabel(str(mainpanel.data.stats['single_channel'][bindex]["rise_time"][index]))
+#        mainpanel.risetimeval.SetLabel(str(mainpanel.data.stats['single_channel'][bindex]["rise_time"][index]))
         mainpanel.risetimeval.SetLabel(str(mainpanel.data.stats['single_channel'][bindex]["rise_time"][index]))
         mainpanel.amplitudeval.SetLabel(str(mainpanel.data.stats['single_channel'][bindex]["amplitude"][index]))
 
@@ -152,8 +231,8 @@ class MainPanel(wx.Panel):
         ctext = wx.StaticText(self, wx.ID_ANY, "Controller Side")
         controlSizer.Add(ctext, 0, wx.CENTER|wx.TOP, 0)
         #Add Analyze Button
-        analyze_button = wx.Button(self, wx.ID_ANY, 'Analyze')
-        controlSizer.Add(analyze_button, 0, wx.ALL, 5)
+        ##analyze_button = wx.Button(self, wx.ID_ANY, 'Analyze')
+        ##controlSizer.Add(analyze_button, 0, wx.ALL, 5)
         #Add Button Sizer to contain 'Previous' and 'Next' Buttons (Hidden on start up)
         buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
         self.next_button = wx.Button(self, wx.ID_ANY, 'Next')
@@ -162,18 +241,18 @@ class MainPanel(wx.Panel):
         self.prev_button.Hide()
         self.Bind(wx.EVT_BUTTON, self.next, self.next_button)
         self.Bind(wx.EVT_BUTTON, self.previous, self.prev_button)
-        self.Bind(wx.EVT_BUTTON, self.analyze, analyze_button)
+        ##self.Bind(wx.EVT_BUTTON, self.analyze, analyze_button)
         buttonSizer.Add(self.prev_button, 0, wx.ALL|wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 5)
         buttonSizer.Add(self.next_button, 0, wx.ALL|wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 5)
         controlSizer.Add(buttonSizer, 0, wx.ALL|wx.EXPAND, 0)
         #Add FileSizer for file input
         fileSizer = wx.BoxSizer(wx.VERTICAL)
         self.filename = wx.StaticText(self, id=wx.ID_ANY, label="No File Currently Selected")
-        browser = wx.Button(self, wx.ID_ANY, 'Browse')
-        browser.Bind(wx.EVT_BUTTON, self.onOpenFile)
+#        browser = wx.Button(self, wx.ID_ANY, 'Browse')
+#        browser.Bind(wx.EVT_BUTTON, self.onOpenFile)
         self.fname = "No File Currently Selected"
         fileSizer.Add(self.filename, 0, wx.ALL, 0)
-        fileSizer.Add(browser, 0, wx.ALL|wx.CENTER, 0)
+#        fileSizer.Add(browser, 0, wx.ALL|wx.CENTER, 0)
         controlSizer.Add(fileSizer, 0, wx.ALL|wx.EXPAND, 0)
         #Create Sizer for Channel Selectors
         self.board1text = wx.StaticText(self, wx.ID_ANY, "Board 1")
@@ -326,7 +405,9 @@ class MainPanel(wx.Panel):
         controlSizer.Add(100,50,0)
         controlSizer.Add(spectrumsizer, 0, wx.ALIGN_CENTER, 0)          
         #Add Feedback Sizer
-        self.fbtext = wx.TextCtrl(self, id=wx.ID_ANY, size=(1000,200), style = wx.TE_READONLY|wx.TE_LEFT)
+        self.fbtext = wx.TextCtrl(self, id=wx.ID_ANY, size=(1000,200), style = wx.TE_READONLY|wx.TE_LEFT|wx.TE_MULTILINE)
+        style = self.fbtext.GetWindowStyle()
+        self.fbtext.AppendText("App Log\n")
         controlSizer.Add(self.fbtext, 0, wx.ALL|wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 0)
         #Add sizers to the main sizer (note the result sizer is blank and will be used when a button is clicked)
         self.mainSizer.Add(controlSizer, 1, wx.ALL, 0)
@@ -335,7 +416,6 @@ class MainPanel(wx.Panel):
         self.SetSizer(self.mainSizer)
         self.mainSizer.Fit(self)
         self.Layout()
-        print(controlSizer.GetSize())
 
     def onOpenFile(self, event):
 #            """
@@ -357,11 +437,14 @@ class MainPanel(wx.Panel):
             self.fname = name
         dlg.Destroy()
         return 
-
+ 
     def analyze(self, event):
         f = self.fname
         #Check to see if Analyze Button Has been clicked yet
         if f != "No File Currently Selected" and f != self.oldfile:
+            #self.fbtext.AppendText("Start: %s" % f)
+            self.next_button.Hide()
+            self.prev_button.Hide()
             self.oldfile = f
             self.changechannelbutton.Hide()
             self.gototext.Hide()
@@ -383,13 +466,14 @@ class MainPanel(wx.Panel):
             self.goto.SetValue(1)
             self.showchannels()
         else:
+            self.fbtext.AppendText("Select Different File\n")
             return
 
 
     def showchannels(self):
         #Iterate through the channels, and display the ones that read_data has available
         self.graphbutton.Show()
-        #multidimensional array that is index by board and then channel
+        #multidimensional array that is indexed by board and then channel
         availablechannels = self.data.loadedchannels
         numchannels = len(availablechannels[0]) + len(availablechannels[1]) 
         if numchannels > 1:
@@ -493,7 +577,7 @@ class MainPanel(wx.Panel):
                         channel2 = self.channel2.GetString(self.channel2.GetSelection())
                         eventfound = False
                         while True:
-                            print("Event not found")
+                            #print("Event not found")
                             ptime1 = float(self.data.stats['single_channel'][int(channel1[-1]) - 1]["peak_time"][self.index])  
                             ptime2 = float(self.data.stats['single_channel'][int(channel2[-1]) - 1]["peak_time"][self.index])  
                             diff = abs(ptime1 - ptime2)
@@ -510,7 +594,7 @@ class MainPanel(wx.Panel):
                                 triggamp = channel.GetChildren()[1].GetWindow().GetValue()
                                 channelnum = int(channel.GetChildren()[0].GetWindow().GetLabel()[8])-1
                                 amp = float(self.data.stats['single_channel'][channelnum]["amplitude"][self.index]) 
-                                print(amp)
+                                #print(amp)
                                 if triggamp > amp * 1000: #Bad
                                     fault = True
                                     break
@@ -548,7 +632,7 @@ class MainPanel(wx.Panel):
       
 
             else:
-                print("No Channel Selected")    
+                self.fbtext.AppendText("No Channel Selected\n")    
                 return
         
 
@@ -571,7 +655,7 @@ class MainPanel(wx.Panel):
                     channel2 = self.channel2.GetString(self.channel2.GetSelection())
                     eventfound = False
                     while True:
-                        print("Event not found")
+                        #print("Event not found")
                         ptime1 = float(self.data.stats['single_channel'][int(channel1[-1]) - 1]["peak_time"][self.index])  
                         ptime2 = float(self.data.stats['single_channel'][int(channel2[-1]) - 1]["peak_time"][self.index])  
                         diff = abs(ptime1 - ptime2)
@@ -588,7 +672,7 @@ class MainPanel(wx.Panel):
                             triggamp = channel.GetChildren()[1].GetWindow().GetValue()
                             channelnum = int(channel.GetChildren()[0].GetWindow().GetLabel()[8])-1
                             amp = float(self.data.stats['single_channel'][channelnum]["amplitude"][self.index]) 
-                            print(amp)
+                            #print(amp)
                             if triggamp > amp * 1000: #Bad
                                 fault = True
                                 break
@@ -604,7 +688,6 @@ class MainPanel(wx.Panel):
                     break                    
                       
 
-            print(self.index)
             axes = self.plotter.add("Event {}".format(self.index + 1)).gca()
             colors = ["Black", "Blue", "Green"]
             i = 0
@@ -668,8 +751,8 @@ class MainPanel(wx.Panel):
      
     def plotwaveformspec(self, event):
         plt.figure()
-        counts, bins = np.histogram(self.data.stats["single_channel"][0]["amplitude"])
-        plt.hist(bins[:-1], bins, weights=counts)
+        amps = self.data.stats["single_channel"][0]["amplitude"]
+        amps.value_counts(normalize = False, sort = False, bins = 250).plot()
         plt.xlabel('Amplitude')
         plt.ylabel('Number of Events')
         plt.title('Frequency vs. Amplitude')
